#!/usr/bin/env node
'use strict';

var argv = require('argv');
var inquirer = require('inquirer');
var javascriptStringify = require('javascript-stringify');
var winston = require('winston');
var wrap = require('word-wrap');
require('util');
var R = require('ramda');
var RA = require('ramda-adjunct');
require('lodash.merge');
var path = require('path');
var findUp = require('find-up');
var configureGit = require('simple-git/promise');
var lodash = require('lodash');
var range = require('node-range');
var toText = require('markdown-to-text');
var pkgDir = require('pkg-dir');
var insertLine = require('insert-line');
var fs = require('fs');
var globOg = require('glob');
var promisify = require('es6-promisify');
var jsonFile = require('jsonfile');
var editor = require('edit-json-file');
var fsExtra = require('fs-extra');
var defaults = require('lodash.defaults');
var cosmiconfig = require('cosmiconfig');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var argv__default = /*#__PURE__*/_interopDefaultLegacy(argv);
var wrap__default = /*#__PURE__*/_interopDefaultLegacy(wrap);
var findUp__default = /*#__PURE__*/_interopDefaultLegacy(findUp);
var configureGit__default = /*#__PURE__*/_interopDefaultLegacy(configureGit);
var range__default = /*#__PURE__*/_interopDefaultLegacy(range);
var toText__default = /*#__PURE__*/_interopDefaultLegacy(toText);
var pkgDir__default = /*#__PURE__*/_interopDefaultLegacy(pkgDir);
var insertLine__default = /*#__PURE__*/_interopDefaultLegacy(insertLine);
var globOg__default = /*#__PURE__*/_interopDefaultLegacy(globOg);
var promisify__default = /*#__PURE__*/_interopDefaultLegacy(promisify);
var jsonFile__default = /*#__PURE__*/_interopDefaultLegacy(jsonFile);
var editor__default = /*#__PURE__*/_interopDefaultLegacy(editor);
var defaults__default = /*#__PURE__*/_interopDefaultLegacy(defaults);

const createWinstonLogger = winston.createLogger,
      transports = winston.transports,
      format = winston.format;
const combine = format.combine,
      timestamp = format.timestamp,
      colorize = format.colorize,
      printf = format.printf;


const cliFormat = printf(({ level, message, timestamp: msgTimestamp }) => `${msgTimestamp} ${level}: ${message}`);

const prettify = obj => `\n${javascriptStringify.stringify(obj, null, '    ')}`;

const createLogger = ({ level }) => {
    const config = logLevel => ({
        format: combine(timestamp(), colorize(), cliFormat),
        transports: [new transports.Console({ level: logLevel })]
    });

    const logger = createWinstonLogger(config(level));
    logger.changeLevel = ({ level: logLevel }) => logger.configure(config(logLevel));

    return logger;
};

var promptSession = (({ logger }) => inquirer.prompt([{
    type: 'number',
    name: 'newSessions',
    message: 'How many sessions are we logging?',
    default: 1,
    validate: value => value > 0
}, {
    type: 'editor',
    name: 'entry',
    message: 'What did you do during this session?'
}]).then(answers => {
    logger.debug(prettify(answers));
    return answers;
}));

R.flip(R.concat);
const toList = R.pipe(RA.list, RA.flattenDepth(1));
R.map(toList);

const prop = R.useWith(R.lensPath, [R.split('.')]);

R.curryN(2, (key = '', value = {}) => {
    if (!RA.isString(key) || !RA.isPlainObject(value)) return value;

    return R.pipe(R.useWith(R.view, [prop]), R.defaultTo(value))(key, value);
});

R.useWith(R.set, [prop]);

const sync = ({ anchor = '.git' } = {}) => path.dirname(findUp__default['default'].sync(anchor));

var patchObject = (patches => R.mapObjIndexed((old, key) => {
    if (!R.has(key, patches)) return old;

    return (...args) => {
        const patch = R.view(R.lensProp(key), patches);
        const result = old(...args);

        return result.then ? result.then(patch) : patch(result);
    };
}));

const camelCaseKeys = obj => {
    if (lodash.isArray(obj)) return obj.map(camelCaseKeys);

    if (lodash.isObject(obj)) {
        const newObj = lodash.mapValues(obj, camelCaseKeys);
        return lodash.mapKeys(newObj, (_, key) => lodash.camelCase(key));
    }

    return obj;
};

const patch = patchObject({ status: camelCaseKeys });
const makeGit = () => patch(configureGit__default['default'](sync()));

makeGit();

var git = makeGit().outputHandler((command, stdout, stderr) => {
    stdout.pipe(process.stdout);
    stderr.pipe(process.stderr);
});

var createCommitter = (({ logger }) => ({
    commit: ({ next, last, entry, configPath, worklogPath }) => {
        const sessions = range__default['default'](next, last + 1).map(_ => _).join('/');
        const headerMessage = `docs(worklog): Log work for session${next !== last ? 's' : ''} ${sessions}.`;
        logger.debug('Commmit Header:');
        logger.debug(`\n${headerMessage}`);

        const sanitizedEntry = toText__default['default'](entry.replace(/\n/g, ' '));
        const entryMessage = wrap__default['default'](sanitizedEntry, { width: 80 });
        logger.debug('Commit Entry:');
        logger.debug(`\n${entryMessage}`);

        return git.add([worklogPath, configPath]).then(() => git.commit([headerMessage, entryMessage])).then(() => logger.info(`Committed sessions: ${sessions}`));
    }
}));

const readFile = jsonFile__default['default'].readFile,
      writeFile = jsonFile__default['default'].writeFile,
      writeFileSync = jsonFile__default['default'].writeFileSync;


const read = promisify__default['default'](readFile);

const writeJson = promisify__default['default'](writeFile);
const write = (file, data) => writeJson(file, data, { spaces: 4 });

const writeSync = (file, data) => writeFileSync(file, data, { spaces: 4 });

const edit = (file, location, data) => {
    const json = editor__default['default'](file, { stringify_width: 4, stringify_eol: true });
    json.set(location, data);
    json.save();
};

var jsonFunctions = {
    read,
    write,
    writeSync,
    edit
};

var _extends = Object.assign || function (target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i];

    for (var key in source) {
      if (Object.prototype.hasOwnProperty.call(source, key)) {
        target[key] = source[key];
      }
    }
  }

  return target;
};

const readStream = fs.createReadStream;
const mkdir = path => fs.mkdirSync(path, { recursive: true });
const readDir = fs.readdirSync;
const exists = fs.existsSync;
const readSync = fs.readFileSync;
const writeSync$1 = fs.writeFileSync;

const lstat = promisify__default['default'](fs.lstat);
const copy = promisify__default['default'](fsExtra.copy);
const read$1 = promisify__default['default'](fs.readFile);
const write$1 = promisify__default['default'](fs.writeFile);

const glob = promisify__default['default'](globOg__default['default']);

const json = jsonFunctions;

const some = { readStream, mkdir, readDir, exists, readSync, writeSync: writeSync$1 };
_extends({}, some, { lstat, copy, read: read$1, write: write$1, glob, json });

var createWorklogger = (({ logger }) => {
    const root = pkgDir__default['default'].sync();
    const worklogFile = path.join(root, 'docs', 'worklog.md');

    return {
        path: () => worklogFile,
        write: ({ next, last, entry }) => {
            logger.debug(`Project Root: ${root}`);
            logger.debug(`Worklog File Path: ${worklogFile}`);

            const sessions = range__default['default'](next, last + 1).map(number => `#${number}`).join(', ');
            logger.info(`Writing worklog for sessions ${sessions}`);

            const content = `## ${sessions}\n\n${entry}`;
            logger.debug('Worklog Entry:');
            logger.debug(`\n${content}`);

            if (!exists(worklogFile)) {
                logger.warn('Worklog does not exist yet, creating one.');
                const initialContent = `# Worklog\n\n${content}`;
                return writeSync$1(worklogFile, initialContent);
            }

            return insertLine__default['default'](worklogFile).contentSync(content).at(3);
        }
    };
});

var createConfigHandler = (({ logger }) => {
    const root = pkgDir__default['default'].sync();
    const explorer = cosmiconfig.cosmiconfigSync('dex', {
        searchPlaces: ['.dexrc'],
        loaders: { noExt: cosmiconfig.defaultLoaders['.json'] }
    });
    let config = explorer.search();

    return {
        path: () => config.filepath,
        get: () => {
            const configDefaults = { session: 0, subprojects: [] };
            if (!config) {
                logger.warn('No Dex config file found');
                return configDefaults;
            }

            logger.info('Config file loaded');
            logger.debug(prettify(config));
            return defaults__default['default'](config.config, configDefaults);
        },
        write: ({ last: session }) => {
            const path$1 = config ? config.filepath : path.join(root, '.dexrc');
            const newConfig = config ? defaults__default['default']({ session }, config.config) : { session };
            writeSync(path$1, newConfig);

            config = explorer.load(path$1);
        }
    };
});

argv__default['default'].version('0.0.23');
argv__default['default'].info('Dex is a Pomodoro inspired worklog system.');

argv__default['default'].option([{
    name: 'log',
    short: 'l',
    type: 'string',
    description: 'Level of logging to print [error, warn, info, verbose, debug, silly]',
    example: '--log=level-name'
}]);

var _argv$run = argv__default['default'].run(),
    _argv$run$options$log = _argv$run.options.log;

const level = _argv$run$options$log === undefined ? 'info' : _argv$run$options$log;

const logger = createLogger({ level });
const configuration = createConfigHandler({ logger });
const worklog = createWorklogger({ logger });
const committer = createCommitter({ logger });

logger.info('Dex CLI');

const config = configuration.get();
logger.info(`${config.session} sessions currently logged.`);

promptSession({ logger }).then(({ newSessions, entry }) => {
    const next = config.session + 1;
    const last = next + newSessions - 1;
    logger.debug(`Next Session: ${next}`);
    logger.debug(`Last Session: ${last}`);

    return { next, last, entry };
}).then(session => {
    worklog.write(session);
    configuration.write(session);
    return _extends({}, session, {
        worklogPath: worklog.path(),
        configPath: configuration.path()
    });
}).then(session => committer.commit(session)).catch(err => {
    logger.error('Error loading config file');
    logger.error(prettify(err));
});
